from django.test import TestCase

from useful.tasks import call_management_command


class ManagementCommandTestCase(TestCase):
    """Test calling a management command as a Celery task"""
    def test_success(self):
        t = call_management_command.delay('validate')
        self.assertEquals(t.status, 'SUCCESS')

    def test_failure(self):
        t = call_management_command.delay('somethingrandomthatdoesntexist')
        self.assertEquals(t.status, 'FAILURE')



def get_object_or_none(model, *args, **kwargs):
    """
    Like get_object_or_404, but doesn't throw an exception.

    Allows querying for an object that might not exist without triggering
    an exception.
    """
    try:
        return model._default_manager.get(*args, **kwargs)
    except model.DoesNotExist:
        return None



import datetime
import json

from django.test import TestCase
from django.http import HttpResponse

from useful.helpers import json_response, jsonp_response


class JsonReponseTestCase(TestCase):
    data = {
        'dict': {
            'a': 'b',
            'c': 'd',
            'e': {'nested': 'dict'},
            'f': ['list', 'as', 'value'],
            'g': ['mixing', {'value': 'types'}],
            'e': u'unicode string',
        },
        'null': None,
        'list': [1, 2, 'a', 'b'],
        'with_time': {'a': 'b', 'time': datetime.datetime.now()}
    }

    def test_json_response(self):
        response = json_response(self.data['dict'])

        self.assertIsNotNone(response)

        self.assertIsInstance(response, HttpResponse)

        self.assertIsNotNone(response.content)
        self.assertIsInstance(response.content, str)

        self.assertEquals(json.loads(response.content), self.data['dict'])

        self.assertEquals(response.status_code, 200)

    def test_list_json_response(self):
        response = json_response(self.data['list'])

        self.assertIsNotNone(response.content)
        self.assertIsInstance(response.content, str)

        self.assertEquals(json.loads(response.content), self.data['list'])

    def test_null_json_response(self):
        response = json_response(self.data['null'])

        self.assertIsNotNone(response.content)
        self.assertIsInstance(response.content, str)

        self.assertEquals(json.loads(response.content), self.data['null'])

    def test_json_response_code(self):
        response = json_response(self.data['null'], status=400)
        self.assertEquals(response.status_code, 400)

    def test_json_serializer(self):
        dthandler = lambda x: (x.isoformat()
                               if isinstance(x, datetime.datetime) else x)

        typerror = lambda x: json_response(self.data['with_time'])

        self.assertRaises(TypeError, typerror)

        response = json_response(self.data['with_time'], serializer=dthandler)

        # Back to datetime object
        t = datetime.datetime.strptime(json.loads(response.content)['time'],
                                       '%Y-%m-%dT%H:%M:%S.%f')

        self.assertEquals(t, self.data['with_time']['time'])


class JsonpReponseTestCase(TestCase):
    """ The same as json_response_test, but can't check the result """

    data = JsonReponseTestCase.data  # Borrow the data from json_response test

    def test_jsonp_response(self):
        response = jsonp_response(self.data['dict'])

        self.assertIsNotNone(response)

        self.assertIsInstance(response, HttpResponse)

        self.assertIsNotNone(response.content)
        self.assertIsInstance(response.content, str)

        self.assertEquals(response.status_code, 200)



from django.http import HttpResponse

try:
    import simplejson as json
except:
    import json


def json_response(data, status=200, serializer=None):
    """
    Returns an HttpResponse object containing JSON serialized data.

    The mime-type is set to application/json, and the charset to UTF-8.
    """
    return HttpResponse(json.dumps(data, default=serializer),
                        status=status,
                        content_type='application/json; charset=UTF-8')


def jsonp_response(data, callback="f", status=200, serializer=None):
    """
    Returns an HttpResponse object containing JSON serialized data,
    wrapped in a JSONP callback.

    The mime-type is set to application/x-javascript, and the charset to UTF-8.
    """
    val = json.dumps(data, default=serializer)
    ret = "{callback}('{val}');".format(callback=callback, val=val)

    return HttpResponse(ret,
                        status=status,
                        content_type='application/x-javascript; charset=UTF-8')



try:
    from django.conf.urls import url, patterns  # Django1.4
except ImportError:
    from django.conf.urls.defaults import url, patterns  # Django1.3

from useful.views import ExtraContextTemplateView


urlpatterns = patterns('',
    url(r'^test_extra_context_view_no_context$',
        ExtraContextTemplateView.as_view(template_name='test.html'),
        name='test_extra_context_view_no_context'),

    url(r'^test_extra_context_view_w_context$',
        ExtraContextTemplateView.as_view(template_name='test.html',
                                         extra_context={'extra': 'context'}),
        name='test_extra_context_view_w_context'),
)



from django.contrib.auth.models import User
from django.test import TestCase

from useful.helpers import get_object_or_none


class GetObjectOrNoneTestCase(TestCase):
    def setUp(self):
        self.user = User.objects.create(username='test',
                                        email='test@example.com')

    def test_get_object(self):
        user = get_object_or_none(User, username='test')
        self.assertIsNotNone(user)
        self.assertEquals(type(user), type(self.user))
        self.assertEquals(user, self.user)

        user = get_object_or_none(User, email='test@example.com')
        self.assertIsNotNone(user)
        self.assertEquals(type(user), type(self.user))
        self.assertEquals(user, self.user)

    def test_get_none(self):
        user = get_object_or_none(User, username='nonexistent')
        self.assertIsNone(user)

        user = get_object_or_none(User, email='doesnot@exist.com')
        self.assertIsNone(user)



from celery.task import task
from django.core import management


@task()
def call_management_command(command):
    """
    Run a management command as a Celery task.

    Useful for running periodic management commands with Celerybeat.
    """

    management.call_command(command)



from setuptools import setup, find_packages

setup(
    name='django-useful',
    author='Yuri Prezument',
    author_email='y@yprez.com',
    version='0.1.0',
    packages=find_packages(),
    license='ISC',
    url='http://github.com/yprez/django-useful',
    description='Reusable Django snippets',
    long_description=open('README.rst').read(),
    include_package_data=True,
    zip_safe=False,
    classifiers=(
        'Development Status :: 2 - Pre-Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: ISC License (ISCL)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ),
)



from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.client import Client


class ExtraContextTemplateViewTestCase(TestCase):
    urls = 'useful.tests.test_urls'

    def setUp(self):
        self.client = Client()

    def test_no_context(self):
        url = reverse('test_extra_context_view_no_context')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name, ['test.html'])
        self.assertFalse('extra' in response.context)

    def test_with_context(self):
        url = reverse('test_extra_context_view_w_context')
        response = self.client.get(url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.template_name, ['test.html'])
        self.assertTrue('extra' in response.context)



from django.views.generic import TemplateView


class ExtraContextTemplateView(TemplateView):
    """
    Extension of Django TemplateView CBV that allows adding extra context
    """
    extra_context = None

    def get_context_data(self, *args, **kwargs):
        context = super(ExtraContextTemplateView, self).\
            get_context_data(*args, **kwargs)
        if self.extra_context:
            context.update(self.extra_context)
        return context
